@@ -10,10 +10,11 @@
    implement the same functionality in your code. It's perhaps 10
    lines.
 
-   Copyright (C) 2003 by Michal Zalewski <lcamtuf@coredump.cx>
+   Copyright (C) 2003-2006 by Michal Zalewski <lcamtuf@coredump.cx>
 
   */
 
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -97,6 +98,8 @@ int main(int argc,char** argv) {
   if (r.link[0]) printf("Link     : %s\n",r.link);
   if (r.tos[0])  printf("Service  : %s\n",r.tos);
 
+  if (r.uptime != -1)  printf("Uptime   : %d hrs\n",r.uptime);
+
   if (r.score != NO_SCORE) 
     printf("M-Score  : %d%% (flags %x).\n",r.score,r.mflags);
 
